"""
module_docstring - checker for vxnorm

This file does not expose an explicite VxChecker object declaration to avoid
dependencies handling, you just need to provide:
    ======================= ===============================================
    parse_file()            Parse the source file
    ======================= ===============================================
"""

#---
# Public
#---

def parse_file(checker, mfile, pathinfo):
    """ parse the mapped file

    The file is mapped using mmap() and seeked through offset 0 to avoid too
    many I/O operations with classical file primitive.

    @args
    > checker (VxChecker) - current checker instance for this file
    > mfile      (VxMmap) - custom mmap instance of the file, seeked at 0
    > pathinfo     (dict) - path information

    @return
    > Nothing
    """
    if pathinfo['filename'] == '__main__.py':
        return

    line = ''
    for line in mfile.getlines():
        if line[0] != '#':
            break
    if line[0] == '#':
        return

    checker.select_rule('modocstr')
    if line == '""""\n':
        checker.notify(0, 'missing docstring')

    modinfo = mfile.readline().split(' ', 2)
    modname = f"{'.'.join(pathinfo['dirs'])}"
    if pathinfo['filename'] != '__init__.py':
        modname += f".{pathinfo['filename'][:-3]}"
    if modinfo[0] != modname:
        checker.notify(
            0,
            f"malformated module name ({modinfo[0]} != {modname})"
        )
        return
    if modinfo[1] != '-':
        checker.notify(0, 'missing separator')
        return

    if mfile.readline() not in ['\n', '"""\n']:
        checker.notify(
            0,
            'missing empty line between declaration and docstring end'
        )
